@@ -1,4 +1,9 @@
-from Queue import Queue, Empty
+from __future__ import unicode_literals
+try:
+    from queue import Queue, Empty
+except ImportError:
+    # Python 2
+    from Queue import Queue, Empty
 from decimal import Decimal
 import json
 import threading
@@ -190,7 +195,7 @@ class Client(object):
         """Runs the reading thread."""
         try:
             while not getattr(self, '_shutdown', False):
-                msg = json.loads(self.conn.recv())
+                msg = json.loads(self.conn.recv().decode('utf-8'))
                 log.debug('<<<<<<<< receiving % s', json.dumps(msg, indent=2))
 
                 type = msg['type']
@@ -212,8 +217,8 @@ class Client(object):
                         continue
 
                 raise ValueError(
-                    u'unexpected message from server: %s' % unicode(msg))
-        except Exception, e:
+                    'unexpected message from server: %s' % str(msg))
+        except Exception as e:
             # If we have already shutdown, ignore the error. This is
             # because by shutting down the socket during a recv(), a
             # variety of socket-related / SSL errors are to be expected.
@@ -241,7 +246,7 @@ class Client(object):
         data = {k:v for k, v in data.items() if v is not None}
 
         log.debug('>>>>>>>> sending %s', json.dumps(data, indent=2))
-        self.conn.send(json.dumps(data, cls=RippleEncoder))
+        self.conn.send(json.dumps(data, cls=RippleEncoder).encode('utf-8'))
         with self.callbacks_lock:
             self.callbacks[data['id']] = DeferredResponse()
         msg = self.callbacks[data['id']].wait()
@@ -402,7 +407,7 @@ class Remote(object):
                             self._pending_transactions[hash].resolve(msg)
                             del self._pending_transactions[hash]
 
-        except Exception, e:
+        except Exception as e:
             # On error, notify all watches
             with self._pending_transactions_lock:
                 for transaction in self._pending_transactions.values():
