@@ -1,5 +1,12 @@
 from decimal import Decimal
 import json
+try:
+    import builtins
+except ImportError:
+    # Python2
+    import __builtin__ as builtins
+import six
+from six.moves import filter
 
 
 # During debugging, this helps with the RipplePrimitive __getattr__
@@ -39,6 +46,7 @@ class RipplePrimitive(dict):
             # (see also the related Python bug report).
             # One possible way to fix this might be implementing
             # __getattribute__ instead and check if a property exists.
+            # Until then, enable the hack on top of this file.
             raise AttributeError(item)
 
     def __repr__(self):
@@ -48,7 +56,10 @@ class RipplePrimitive(dict):
         return json.dumps(self.__json__())
 
     def __json__(self):
-        return self.data
+        return dict.copy(self)
+
+    def copy(self):
+        return RipplePrimitive(self)
 
 
 class tupledict(list):
@@ -101,9 +112,10 @@ class RippleStateEntry(RipplePrimitive):
 
 class Amount(RipplePrimitive):
 
-    def __init__(self, data):
+    def __init__(self, data, safe=False):
         if isinstance(data, dict):
             # Proper currency structure
+            data['value'] = Decimal(data['value'])
             RipplePrimitive.__init__(self, data)
         else:
             # Parse the amount
@@ -112,9 +124,11 @@ class Amount(RipplePrimitive):
             if isinstance(data, int):
                 # A raw xrp number in drops:
                 value = xrp(data)
-            elif isinstance(data, basestring):
+            elif isinstance(data, six.string_types):
                 if '.' in data:
                     value = Decimal(data)
+                elif safe:
+                    value = xrp(data)
                 else:
                     # For safety, so there can be no confusion.
                     raise ValueError('When passing a string as amount, it '
@@ -140,6 +154,41 @@ class Amount(RipplePrimitive):
         else:
             return RipplePrimitive.__json__(self)
 
+    def __unicode__(self):
+        return '%s' % self.value
+
+    def _assert_compat_other(self, other):
+        """For arithmetic with the Amount class, check that the ``other``
+        object can be handled.
+        """
+        if isinstance(other, unicode):
+            return Decimal(unicode)
+        if isinstance(other, Decimal):
+            return other
+        assert other.currency == self.currency
+        return other['value']
+
+    def __add__(self, other):
+        other_value = self._assert_compat_other(other)
+        result = self.copy()
+        result['value'] = self['value'] + other_value
+        return result
+
+    def __sub__(self, other):
+        other_value = self._assert_compat_other(other)
+        result = self.copy()
+        result['value'] = self['value'] - other_value
+        return result
+
+    def __div__(self, other):
+        other_value = self._assert_compat_other(other)
+        result = self.copy()
+        result['value'] = self['value'] / other_value
+        return result
+
+    def __rdiv__(self, other):
+        return Amount.__truediv__(other, self)
+
 
 
 class AccountRootEntry(RipplePrimitive):
@@ -276,7 +325,11 @@ class Transaction(RipplePrimitive):
         """Return affected nodes matching the filters."""
         result = self.affected_nodes
         if account:
-            result = filter(lambda n: n.affects_account(account), result)
+            # Allow to filter by multiple accounts
+            if not isinstance(account, list):
+                account = [account]
+            result = filter(
+                lambda n: all([n.affects_account(a) for a in account]), result)
         if type:
             result = filter(lambda n: n.type == type, result)
         return list(result)
@@ -385,9 +438,13 @@ class PaymentTransaction(Transaction):
         """3-tuple of (amount, currency, issuers), representing the full
         amount received.
         """
+        if self.meta.get('DeliveredAmount'):
+            amount = self.meta.DeliveredAmount.value
+        else:
+            amount = self.Amount.value \
+                if isinstance(self.Amount, dict) else xrp(self.Amount)
         return tuple(
-            [self.Amount.value if isinstance(self.Amount, dict) else xrp(self.Amount)] +
-            list(self.currencies_received)
+            [amount] + list(self.currencies_received)
         )
 
     @property
@@ -505,7 +562,7 @@ class PaymentTransaction(Transaction):
         """
 
         # list() used once exhausts the generator, it's tiring..
-        filter = lambda f, d: list(__builtins__['filter'](f, d))
+        filter = lambda f, d: list(builtins.filter(f, d))
 
         # Ignore all DirectoryNodes, not sure what they do, it seems
         # like upkeep.
